package com.wincornixdorf.vnc.gui;

import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.security.Principal;
import java.util.Observer;
import java.util.Observable;

import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLSession;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import com.wincornixdorf.vnc.global.VNCViewer;
import com.wincornixdorf.vnc.rfb.VNCSession;
import com.wincornixdorf.vnc.rfb.RFBHandler;
import com.wincornixdorf.vnc.rfb.RFBHandlerEvent;
import com.wincornixdorf.vnc.rfb.conf.VNCSessionCfg;

public class SessionStatusPanel extends JPanel implements Observer, ActionListener {
    private static final long serialVersionUID = 8998828959316040163L;
    private final String        stPref          = "State: ";
    private final String        hiPref          = "Host: ";
    private final String        na              = "<Not available>";
    private final String        NO_PEER_INFO    = "No peer information available!";

    private final ImageIcon     iFB             = new ImageIcon( this.getClass().getResource("/16x16/desktop22.png"));
    private final ImageIcon     iFT             = new ImageIcon( this.getClass().getResource("/16x16/file-manager.png"));
    private final ImageIcon     iHI             = new ImageIcon( this.getClass().getResource("/16x16/server.png"));

    private final ImageIcon     iSecure         = new ImageIcon( this.getClass().getResource("/16x16/encrypted.png"));
    private final ImageIcon     iInsecure       = new ImageIcon( this.getClass().getResource("/16x16/decrypted.png"));
    private final ImageIcon     iNoPeerInfo     = new ImageIcon( this.getClass().getResource("/16x16/encryptedNoCert.png"));
    private final ImageIcon     iInvalid        = new ImageIcon( this.getClass().getResource("/16x16/lock.png"));

    // The session being observed
    private VNCSession          s               = null;

    private JLabel              lFB             = null; // Status Framebuffer connection
    private JLabel              lFT             = null; // Status Filetransfer connection
    private JLabel              lHI             = null; // Status Host Information

    private JLabel              lFBSSL          = null; // Status SSL Framebuffer connection
    private JLabel              lFTSSL          = null; // Status SSL Filetransfer connection

    public SessionStatusPanel() {
        super( new GridLayout( 1, 3));
        init();
    }

    public SessionStatusPanel( VNCSession sIn) {
        super( new GridLayout( 1, 3));
        init();
        setSession( sIn);
    }

    public void setSession( VNCSession sIn) {
        if ( s != null || sIn == null) return;

        s = sIn;

        RFBHandler fbConn = s.getFB();
        RFBHandler ftConn = s.getFT();

        if ( fbConn != null) fbConn.addObserver( this);
        if ( ftConn != null) ftConn.addObserver( this);

        VNCSessionCfg cfg = s.getConfiguration();
        if ( cfg == null) return;

        // Display the servers hostame/port
        lHI.setText( cfg.vncserver.isRemoteSupport() ? cfg.vncserver.getHostParams().getOriginalHostname() : cfg.vncserver.getHostnameAndPort( false, false));

        // Replace the default icons indicating a certain connection is not
        // available with the appropriate ones.
        if ( cfg.cn.isFBPossible()) lFB.setIcon( iFB);
        if ( cfg.cn.isFTPossible()) lFT.setIcon( iFT);
    }

    private void init() {
        lFB = new JLabel( stPref + na, iInvalid, SwingConstants.LEFT);
        lHI = new JLabel( hiPref + na, iHI,      SwingConstants.CENTER);
        lFT = new JLabel( stPref + na, iInvalid, SwingConstants.RIGHT);

        lFBSSL = new JLabel( iInsecure);
        lFTSSL = new JLabel( iInsecure);
        lFBSSL.setToolTipText( NO_PEER_INFO);
        lFTSSL.setToolTipText( NO_PEER_INFO);
        
        JPanel pFB = new JPanel( new FlowLayout( FlowLayout.LEFT, 3, 3));
        JPanel pFT = new JPanel( new FlowLayout( FlowLayout.RIGHT, 3, 3));

        pFB.add( lFBSSL);
        pFB.add( lFB);

        pFT.add( lFTSSL);
        pFT.add( lFT);

        add( pFB);
        add( lHI);
        add( pFT);
    }

    private void updateStatusLabel( RFBHandler c, RFBHandlerEvent e) {
        if ( e == null || e.operation != RFBHandlerEvent.Category.PHASECHANGED) return;

        JLabel l = c.canHandleFileTransfers() ? lFT : lFB;
        l.setText( stPref + e.nyu.toString());
    }

    private String getTTT( SSLSession s, Principal p) {
        if ( p == null) return NO_PEER_INFO;

        StringWriter sw = new StringWriter( 1024);
        PrintWriter pw = new PrintWriter( sw);
        pw.println( "<html>");
        pw.println( "<ul>");
        pw.printf(  "<li>PeerName : [%1$s]</li>", p.getName());
        pw.printf(  "<li>Protocol : [%1$s]</li>", s.getProtocol());
        pw.printf(  "<li>Cipher   : [%1$s]</li>", s.getCipherSuite());
        pw.println( "</ul>");
        pw.println( "</html>");

        return sw.toString();
    }

    private void updateSSLSessionLabel( RFBHandler c) {
        JLabel      l   = c.canHandleFileTransfers() ? lFTSSL : lFBSSL;
        SSLSession  s   = c.getSSLSession();
        ImageIcon   ico =  iInsecure;
        String      ttt = NO_PEER_INFO;

        if ( c.isEncrypted()) {
            ico = c.isTrustingAll() ? iNoPeerInfo : iSecure;

            try {
                Principal p = s.getPeerPrincipal();
                ttt = getTTT( s, p);
            } catch ( SSLPeerUnverifiedException e) {
                ico = iNoPeerInfo;
            }
        }

        l.setIcon( ico);
        l.setToolTipText( ttt);
    }

    public boolean isNecessary() {
    	if ( s == null) return false;

    	if ( s.hasBothConnectionsDown()) return false;
    	if ( s.hasOpenConnection( true)  && !s.getFB().hasVerifiedPeer()) return true;   	
    	if ( s.hasOpenConnection( false) && !s.getFT().hasVerifiedPeer()) return true;
    	
    	return false;
    }

    private void hideIfUnneccessary() {
    	boolean neccessary = isNecessary();
    	setVisible( neccessary);    	
    }

    public void update( Observable o, Object arg) {
        if (o == null || arg == null || !( arg instanceof RFBHandlerEvent)) return;

        final ActionListener  l = this;          
        final RFBHandler      c = (RFBHandler)o;
        final RFBHandlerEvent e = (RFBHandlerEvent)arg;
		
        
        if ( e.operation != RFBHandlerEvent.Category.PHASECHANGED) return;
        VNCViewer.pen.finest( "SessionStatusPanel forced to update " + e.old + " -> " + e.nyu);
        
        try {
            SwingUtilities.invokeAndWait( new Runnable() {
                public void run() {
                 // Unlikely, but better safe than sorry!
                    //if ( c == null) return;
                    
                    updateStatusLabel( c, e);
                    updateSSLSessionLabel( c);
                    
                    // Starting a timer there triggers an Action Event, designed to hide
                    // this widget unless the connection is not perfectly secure.
                    Timer t = new Timer( 5000, l);
                    t.setRepeats( false);
                    t.start();                    
                }
            });
        } catch ( Exception ex) { }
    }

    @Override
    public void setVisible( boolean aFlag) {
    	super.setVisible( aFlag || isNecessary());
    }

	@Override
	public void actionPerformed( ActionEvent arg0) {
		hideIfUnneccessary();
	}
}
