package com.wincornixdorf.vnc.rfb;

import java.nio.charset.Charset;

/**This class is used to conveniently store constants related to the RFB protocol.*/
public class RFB {

	// Repeaters will always identify themselves by sending this message
    public final static String  REPEATER_IDENTIFIER	= "RFB 000.000\n";

    public final static String  VERSIONMSG_3_3 		= "RFB 003.003\n",
                                VERSIONMSG_3_6 		= "RFB 003.006\n",
                                VERSIONMSG_3_7 		= "RFB 003.007\n",
                                VERSIONMSG_3_8 		= "RFB 003.008\n";

    /**This is the Charset raw bytes are expected to be encoded in when they
     * have to be interpreted as a String. This encoding will ***NOT*** be
     * applied when decoding RFB.ServerCutText messages!**/
    public       static Charset DEFAULT_CHARSET  = Charset.forName( "US-ASCII");
    
    public static enum AuthType {
        INVALID( 0, Integer.MAX_VALUE),
        NONE( 1, 1),
        VNCPASSWORD( 2, 2),
        RA2( 5, 99),
        RA2NE( 6, 99),
        TIGHT( 16, 90),
        ULTRA( 17, 80),
        TLS( 18, 99),
        VENCRYPT( 19, 99),
        GTKVNCSASL( 20, 99),
        MD5HASH( 21, 99),
        COLINDEAN( 22, 99),
        MSLOGONII( 113, 3),
        MSLOGON( 0xfffffffa, 4),
        MAC_OSX_SECTYPE_30( 30, 99),
        MAC_OSX_SECTYPE_35( 35, 99);

        private final int val;
        private final int rank;
        
        public static AuthType byValue( byte b) {
            for ( AuthType t : AuthType.values())
                if ( t.asByte() == b) return t;
            return INVALID;
        }
        
        private AuthType( int val, int rank) {
            this.val  = val;
            this.rank = rank;
        }
        
        public byte asByte() { return (byte)val; }
        public int asInt() { return val; }
        
        public boolean isSupported() {
            return this == NONE || this == VNCPASSWORD ||
                   this == MSLOGON || this == MSLOGONII;
        }

        public boolean isPreferable( AuthType to) {
            return this.rank < to.rank; 
        }

        public String toDbgStr() {
            String s = "REALLY unknown";
            
            switch ( this) {
                case INVALID        	: s = "Invalid";            	break;
                case NONE           	: s = "No";                 	break;
                case VNCPASSWORD    	: s = "Plain Password";     	break;
                case RA2            	: s = "RA2";                	break;
                case RA2NE          	: s = "RA2ne";              	break;
                case TIGHT          	: s = "Tight";              	break;
                case ULTRA          	: s = "UltraVNC Secure";    	break;
                case TLS            	: s = "TLS";                	break;
                case VENCRYPT       	: s = "VeNCrypt";           	break;
                case GTKVNCSASL     	: s = "GTK-VNC-SASL";       	break;
                case MD5HASH        	: s = "MD5 Hash";           	break;
                case COLINDEAN      	: s = "Colin Dean";         	break;
                case MSLOGONII      	: s = "MS-Logon II";        	break;
                case MSLOGON        	: s = "MS-Logon";           	break;
                case MAC_OSX_SECTYPE_30	: s = "MAC-OS X Sectype 30";	break;
                case MAC_OSX_SECTYPE_35 : s = "MAC-OS X Sectype 35";	break;
                default:
                	break;
            }

            return  "[" + this.val + "] " + s +" Authentification";
        }
    }

    public static enum AuthResult {
        OK( 0),
        FAILED( 1),
        TOOMANY( 2);
        
        private final int val;

        public static AuthResult byValue( int i) {
            for ( AuthResult r : AuthResult.values())
                if ( r.val == i) return r;
            return FAILED;
        }
        
        private AuthResult( int val) {
            this.val  = val;
        }
    }
    
    // VNC authentication results
//    public final static int     VncAuthOK                   = 0,
//                                VncAuthFailed               = 1,
//                                VncAuthTooMany              = 2;

    public final static int     FramebufferUpdate           = 0,
                                SetColourMapEntries         = 1,
                                Bell                        = 2,
                                ServerCutText               = 3,
                                rfbFileTransfer             = 7;

    // Client-to-server messages
    public final static int     SetPixelFormat              = 0,
                                FixColourMapEntries         = 1,
                                SetEncodings                = 2,
                                FramebufferUpdateRequest    = 3,
                                KeyboardEvent               = 4,
                                PointerEvent                = 5,
                                ClientCutText               = 6,
                                ServerInputUpdateRequest    = 9;

    public final static int     sz_rfbFileTransferMsg       = 12, // FileTransfer Content types and Parameter defines
                                rfbDirContentRequest        =  1, // Client asks for the content of a given Server directory
                                rfbDirPacket                =  2, // Full directory name or full file name. Null content means end of Directory
                                rfbFileTransferRequest      =  3, // Client asks the server for the transfer of a given file
                                rfbFileHeader               =  4, // First packet of a file transfer, containing file's features
                                rfbFilePacket               =  5, // One slice of the file
                                rfbEndOfFile                =  6, // End of file transfer (the file has been received or error)
                                rfbAbortFileTransfer        =  7, // The file transfer must be aborted, whatever the state
                                rfbFileTransferOffer        =  8, // The client offers to send a file to the server
                                rfbFileAcceptHeader         =  9, // The server accepts or rejects the file
                                rfbCommand                  = 10, // The Client sends a simple command (File Delete, dir create etc...)
                                rfbCommandReturn            = 11, // New FT Protocol (V2) The zipped checksums of the destination file (Delta Transfer)
                                rfbFileChecksums            = 12, // The Client receives the server's answer about a simple command

                                // rfbDirContentRequest client Request - content parameters 
                                rfbRDirContent              = 1, // Request a Server Directory contents
                                rfbRDrivesList              = 2; // Request the server's drives list

    public final static int     EncodingidAuto              = -1,   // A fake ID making things easier
                                EncodingRaw                 = 0,
                                EncodingCopyRect            = 1,
                                EncodingRRE                 = 2,
                                EncodingCoRRE               = 4,
                                EncodingHextile             = 5,
                                EncodingZlib                = 6,
                                EncodingTight               = 7,
                                EncodingZRLE                = 16,
                                EncodingCompressLevel0      = 0xFFFFFF00,
                                EncodingQualityLevel0       = 0xFFFFFFE0,
                                EncodingXCursor             = 0xFFFFFF10,
                                EncodingRichCursor          = 0xFFFFFF11,
                                EncodingPointerPos          = 0xFFFFFF18,
                                EncodingLastRect            = 0xFFFFFF20,
                                EncodingNewFBSize           = 0xFFFFFF21,

                                // rfbDirPacket & rfbCommandReturn  server Answer - content parameters
                                rfbADirectory               = 1, // Reception of a directory name
                                rfbAFile                    = 2, // Reception of a file name 
                                rfbADrivesList              = 3, // Reception of a list of drives
                                rfbADirCreate               = 4, // Response to a "create dir" command 
                                rfbADirDelete               = 5, // Response to a "delete dir" command 
                                rfbAFileCreate              = 6, // Response to a "create file" command 
                                rfbAFileDelete              = 7, // Response to a "delete file" command

                                // rfbCommand Command - content parameters
                                rfbCDirCreate               = 1, // Request the server to create the given directory
                                rfbCDirDelete               = 2, // Request the server to delete the given directory
                                rfbCFileCreate              = 3, // Request the server to create the given file
                                rfbCFileDelete              = 4; // Request the server to delete the given file

    // Error when a command fails on remote side (ret in "size" field)
    public final static int     sz_rfbBlockSize             = 8192; // new FT protocol (v2)

    public final static int     MaxNormalEncoding           = 255;

    // Constants used in the Hextile decoder
    public final static int     HextileRaw                  = 1,
                                HextileBackgroundSpecified  = 2,
                                HextileForegroundSpecified  = 4,
                                HextileAnySubrects          = 8,
                                HextileSubrectsColoured     = 16;

    // Constants used in the Tight decoder
    public final static int     TightMinToCompress          = 12;

    public final static int     TightExplicitFilter         = 0x04,
                                TightFill                   = 0x08,
                                TightJpeg                   = 0x09,
                                TightMaxSubencoding         = 0x09,
                                TightFilterCopy             = 0x00,
                                TightFilterPalette          = 0x01,
                                TightFilterGradient         = 0x02;
}
