package com.wincornixdorf.vnc.rfb;

import java.util.Observable;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;

import javax.swing.KeyStroke;

/**
 * A RFBConnJob represents a task which is to be executed by a VNCConnection.
 * Developers should construct a Job using one of the static members below
 * and place the request on a queue which gets processed by the VNCConnection
 * on its thread.
 */
public class RFBConnJob extends Observable {
    public enum Category {
        NONE(null), /**
         * Merely a fallback.
         */
        CONNECT(CategoryType.FRAME), /**
         * Open the connection.
         */
        DISCONNECT(CategoryType.FRAME), /**
         * Close the connection.
         */
        TERMINATE_THREAD(CategoryType.FRAME), /**
         * Kill the thread controlling the connection.
         */
        TRANSFER(CategoryType.TRANSFER), /**
         * Up-/Download a file or directory.
         */
        DELETEFILE(CategoryType.TRANSFER), /**
         * Delete a file or directory.
         */
        CREATEDIR(CategoryType.TRANSFER), /**
         * Create a folder.
         */
        CHANGEDIR(CategoryType.TRANSFER), /**
         * Change to a directory on the server.
         */
        LISTDRIVES(CategoryType.TRANSFER), /**
         * Get a list of available root nodes.
         */
        REFRESH(CategoryType.FRAME), /**
         * Request FB update
         */
        SHUTDOWNCONSOLE(CategoryType.CONSOLE), /**
         * Request the servers console to be turned off.
         */
        POWERUPCONSOLE(CategoryType.CONSOLE), /**
         * Request the servers console to be turned on.
         */
        MOUSE(CategoryType.CONTROLS), /**
         * Send a mouse event.
         */
        TOGGLEJAMALT(CategoryType.CONTROLS), /**
         * Send an Alt key press event WITHOUT the corresponding release event. (or vice versa)
         */
        KEYSTROKE(CategoryType.CONTROLS), /**
         * Send an arbitrary Keystroke event.
         */
        KEYPRESS(CategoryType.CONTROLS), /**
         * Send a keypress event.
         */
        MONKEY(CategoryType.CONTROLS), /**
         * Send a Ctrl Alt Delete Key event.
         */
        META(CategoryType.CONTROLS), /**
         * Send a Windumb Key event.
         */
        CLIPBOARD(CategoryType.CONTROLS), /**
         * Send data to servers clipboard.
         */
        REENCODE(CategoryType.FRAME), /**
         * Request using a different RFB encoding.
         */
        PIXFORMAT(CategoryType.FRAME);

        private CategoryType categoryType;

        Category(CategoryType categoryType) {
            this.categoryType = categoryType;
        }

        public boolean isFrameType() {
            return categoryType != null && categoryType.equals(CategoryType.FRAME);
        }

        public boolean isControlsType() {
            return categoryType != null && categoryType.equals(CategoryType.CONTROLS);
        }

        public boolean isTransferType() {
            return categoryType != null && categoryType.equals(CategoryType.TRANSFER);
        }

        public boolean isConsoleType() {
            return categoryType != null && categoryType.equals(CategoryType.CONSOLE);
        }

        private enum CategoryType {
            CONTROLS,
            TRANSFER,
            FRAME,
            CONSOLE
        }

    }


    public RFBHandler assignedTo = null;

    /**
     * The kind of operation this Event is referring to.
     */
    public Category operation = Category.NONE;

    /**
     * File or directory being Downloaded or unzipped.
     */
    public String srcFile = "";

    /**
     * Destination as to where the above source is Downloaded or unzipped to.
     */
    public String dstFile = "";

//     public String fakeFile              = "";

    /**
     * Optional error message.
     */
    public String errMsg = "";

    /**
     * The operations progress in percent.
     */
    public int progress = 0;

    /**
     * If set true, a File or Directory is being downloaded. If false, it is
     * being uploaded.
     */
    public boolean isDownload = false;

    /**
     * If set true, a File or Directory is being DEflated. If false, it is
     * being INflated.
     */
    public boolean isDeflating = false;

    /**
     * If set the source is a Directory. If false, it is a file.
     */
    public boolean isDirectory = false;

    /**
     * If set the remote cursors pixel data has changed. If false it was the
     * remote desktops pixel data.
     */
    public boolean isMouse = false;

    /**
     * Will be false until the operation was successfully completed.
     */
    public boolean success = false;

    /**
     * In the event of a Down- Upload of a file or directory this is the bits
     * sent/received during the operation. This is of course by no means an exact
     * figure and will at best impress the mayor but no one else.
     */
    public long bps = 0;

    public Component component = null;
    public MouseEvent mouseEvent = null;
    public KeyEvent keyEvent = null;
    public KeyStroke keyStroke = null;

    public static RFBConnJob connect(RFBHandler o) {
        return new RFBConnJob(o, Category.CONNECT);
    }

    public static RFBConnJob disconnect(RFBHandler o) {
        return new RFBConnJob(o, Category.DISCONNECT);
    }

    public static RFBConnJob terminateThread(RFBHandler o) {
        return new RFBConnJob(o, Category.TERMINATE_THREAD);
    }

    public static RFBConnJob transfer(RFBHandler o,
                                      String src,
                                      String dst,
                                      boolean isDownload,
                                      boolean isDirectory) {
        RFBConnJob j = new RFBConnJob(o, Category.TRANSFER);
        j.srcFile = src;
        j.dstFile = dst;
        j.isDownload = isDownload;
        j.isDirectory = isDirectory;
        return j;
    }

    public static RFBConnJob deleteFile(RFBHandler o, String file2delete) {
        RFBConnJob j = new RFBConnJob(o, Category.DELETEFILE);
        j.srcFile = file2delete;
        j.isDirectory = false;
        return j;
    }

    public static RFBConnJob createDirectory(RFBHandler o, String dirName) {
        RFBConnJob j = new RFBConnJob(o, Category.CREATEDIR);
        j.srcFile = dirName;
        j.isDirectory = true;
        return j;
    }

    public static RFBConnJob changeDir(RFBHandler o, String dir, boolean absolute) {
        RFBConnJob j = new RFBConnJob(o, Category.CHANGEDIR);
        j.isDirectory = absolute;
        j.dstFile = dir;
        return j;
    }

    public static RFBConnJob listDrives(RFBHandler o) {
        return new RFBConnJob(o, Category.LISTDRIVES);
    }

    public static RFBConnJob refresh(RFBHandler o) {
        return new RFBConnJob(o, Category.REFRESH);
    }

    public static RFBConnJob turnConsoleOff(RFBHandler o) {
        return new RFBConnJob(o, Category.SHUTDOWNCONSOLE);
    }

    public static RFBConnJob turnConsoleOn(RFBHandler o) {
        return new RFBConnJob(o, Category.POWERUPCONSOLE);
    }

    public static RFBConnJob moveMouse(RFBHandler o, MouseEvent e) {
        RFBConnJob j = new RFBConnJob(o, Category.MOUSE);
        j.mouseEvent = e;
        return j;
    }

    public static RFBConnJob jamAltKey(RFBHandler o) {
        RFBConnJob j = new RFBConnJob(o, Category.TOGGLEJAMALT);
        return j;
    }

    public static RFBConnJob pressKey(RFBHandler o, KeyEvent e) {
        RFBConnJob j = new RFBConnJob(o, Category.KEYPRESS);
        j.keyEvent = e;
        return j;
    }

    public static RFBConnJob pressKeyStroke(RFBHandler o, KeyStroke kse, Component c) {
        RFBConnJob j = new RFBConnJob(o, Category.KEYSTROKE);
        j.keyStroke = kse;
        j.component = c;
        return j;
    }

    public static RFBConnJob pressMonkeyKey(RFBHandler o, Component c) {
        RFBConnJob j = new RFBConnJob(o, Category.MONKEY);
        j.component = c;
        return j;
    }

    public static RFBConnJob pressMetaKey(RFBHandler o, Component c) {
        RFBConnJob j = new RFBConnJob(o, Category.META);
        j.component = c;
        return j;
    }

    public static RFBConnJob setClipboard(RFBHandler o, String contents) {
        RFBConnJob j = new RFBConnJob(o, Category.CLIPBOARD);
        j.srcFile = contents;
        return j;
    }

    public static RFBConnJob reencode(RFBHandler o) {
        return new RFBConnJob(o, Category.REENCODE);
    }

    public static RFBConnJob changePixelFormat(RFBHandler o) {
        return new RFBConnJob(o, Category.PIXFORMAT);
    }

    private RFBConnJob(RFBHandler source) {
        assignedTo = source;
    }

    private RFBConnJob(RFBHandler source, Category c) {
        assignedTo = source;
        operation = c;
    }

    public boolean mayBeQueuedAt(boolean aFBConnection) {
        switch (operation) {
            case NONE:
                return false;
            case CONNECT:
                return true;
            case DISCONNECT:
                return true;
            case TERMINATE_THREAD:
                return true;
            case TRANSFER:
                return !aFBConnection;
            case DELETEFILE:
                return !aFBConnection;
            case CREATEDIR:
                return !aFBConnection;
            case CHANGEDIR:
                return !aFBConnection;
            case LISTDRIVES:
                return !aFBConnection;
            case REFRESH:
                return aFBConnection;
            case SHUTDOWNCONSOLE:
                return aFBConnection;
            case POWERUPCONSOLE:
                return aFBConnection;
            case MOUSE:
                return aFBConnection;
            case TOGGLEJAMALT:
                return aFBConnection;
            case KEYPRESS:
                return aFBConnection;
            case MONKEY:
                return aFBConnection;
            case KEYSTROKE:
                return true;
            case META:
                return aFBConnection;
            case CLIPBOARD:
                return aFBConnection;
            case REENCODE:
                return aFBConnection;
            case PIXFORMAT:
                return aFBConnection;
            default:
                return false;
        }
    }

    public boolean requiresWorkingConnection() {
        switch (operation) {
            case CONNECT:
                return false;
            case DISCONNECT:
                return false;
            case TERMINATE_THREAD:
                return false;
            default:
                return true;
        }
    }

    public void dump() {
        System.out.println("------------ VNCJob ------------");
        System.out.println("============================================");
        System.out.println("Type        : [" + operation + "]");
        System.out.println("Source      : [" + srcFile + "]");
        System.out.println("Destination : [" + dstFile + "]");
        System.out.println("isDownload  : [" + isDownload + "]");
        System.out.println("isDirectory : [" + isDirectory + "]");
        System.out.println("isDeflating : [" + isDeflating + "]");
        System.out.println("isMouse     : [" + isMouse + "]");
        System.out.println("success     : [" + success + "]");
        System.out.println("progress    : [" + progress + "]");
        System.out.println("bps         : [" + bps + "]");
        System.out.println("error Msg   : [" + errMsg + "]");
    }

}
