package com.wincornixdorf.vnc.rfb;

import java.awt.event.KeyEvent;
import java.io.PrintStream;
import java.util.EventObject;

public class RFBHandlerEvent extends EventObject {
    private static final long serialVersionUID = -2903254081996254352L;

    public enum Category {
        NONE, /**
         * Merely a fallback.
         */
        CONNECT, /**
         * Connection was (re)opened.
         */
        DISCONNECT, /**
         * Connection was closed.
         */
//        STATUSCHANGED,  /**The connections internal status flag changed. See cursorX and cursorY for more information.*/
        PHASECHANGED, /**
         * The connection left a phase and entered a new one.
         */
        KEYJAMCHANGED, /**
         * A key was "jammed" or released. See success.
         */
        PDCHANGED, /**
         * The pixel data of the remote desktop or cursor has changed.
         */
        FTBUSY, /**
         * A FT operation has been started / finished. See success.
         */
        FTBEGIN, /**
         * Up- or Download of a File or a directory was initiated.
         */
        FTPROGRESS, /**
         * Up- or Download of a File or a directory has progressed.
         */
        FTERROR, /**
         * Upload of file is not possible. Fe: insufficient privileges
         */
        FTEND, /**
         * Up- or Download of a File or a directory was completed.
         */
        ZIPBEGIN, /**
         * In- or deflating of a (Zip)File has begun.
         */
        ZIPPROGRESS, /**
         * In- or deflating of a (Zip)File has progressed.
         */
        ZIPEND, /**
         * In- or deflating of a (Zip)File has ended.
         */
        DRVLISTCHANGED, /**
         * The list of available drives changed.
         */
        CDBEGIN, /**
         * Sending request to change remote servers directory.
         */
        CDEND, /**
         * Request to change remote servers directory is complete.
         */
        RFSGREW, /**
         * Server sent a file- or directory name which was added to the RFS list.
         */
        DIRCREATE, /**
         * A directory was created on the server.
         */
        FILEDELETE, /**
         * A file [or directory] was deleted on the server.
         */
        CLIPCHANGED, /**
         * The servers clipboard contents changed.
         */
        CURSORMOVED, /**
         * The server requested the mouse pointer to be moved.
         */
        CONSOLESHUTDOWN /**The servers remote console was requested to be powered up or down. See success*/
    }

    /**
     * The kind of operation this Event is referring to.
     */
    public Category operation = Category.NONE;

    /**
     * File or directory being Downloaded or unzipped.
     */
    public String srcDownloadOrUnzip = "";

    /**
     * Destination as to where the above source is Downloaded or unzipped to.
     */
    public String dstDownloadOrUnzip = "";

    /**
     * Optional error message.
     */
    public String errMsg = "";

    /**
     * The operations progress in percent.
     */
    public int progress = 0;

    /**
     * If set true, a File or Directory is being downloaded. If false, it is
     * being uploaded.
     */
    public boolean isDownload = false;

    /**
     * If set true, a File or Directory is being DEflated. If false, it is
     * being INflated.
     */
    public boolean isDeflating = false;

    /**
     * If set the source is a Directory. If false, it is a file.
     */
    public boolean isDirectory = false;

    /**
     * If set the remote cursors pixel data has changed. If false it was the
     * remote desktop's pixel data.
     */
    public boolean isMouse = false;

    /**
     * Will be false until the operation was successfully completed.
     */
    public boolean success = false;

    /**
     * In the event of a Down- Upload of a file or directory this is the bits
     * sent/received during the operation. This is of course by no means an exact
     * figure and will at best impress the mayor but no one else.
     */
    public long bps = 0;

    /**
     * Cursor Move events will use this as the new X coordinate.
     */
    public int cursorX = 0;

    /**
     * Cursor Move events will use this as the new Y coordinate.
     */
    public int cursorY = 0;

    public RFBHandler.Phase old = RFBHandler.Phase.UNKNWON;
    public RFBHandler.Phase nyu = RFBHandler.Phase.UNKNWON;

    public static RFBHandlerEvent connect(RFBHandler o) {
        return new RFBHandlerEvent(o, Category.CONNECT);
    }

    public static RFBHandlerEvent disconnect(RFBHandler o, boolean intentionally) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.DISCONNECT);
        e.success = intentionally;
        return e;
    }

    public static RFBHandlerEvent phasechanged(RFBHandler o,
                                               RFBHandler.Phase op, RFBHandler.Phase np) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.PHASECHANGED);
        e.old = op;
        e.nyu = np;
        return e;
    }

    public static RFBHandlerEvent clipchanged(RFBHandler o) {
        return new RFBHandlerEvent(o, Category.CLIPCHANGED);
    }

    public static RFBHandlerEvent consoleshutdown(RFBHandler o) {
        return new RFBHandlerEvent(o, Category.CONSOLESHUTDOWN);
    }

    public static RFBHandlerEvent cdbegin(RFBHandler o,
                                          String presentDir,
                                          String futureDir) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.CDBEGIN);
        e.srcDownloadOrUnzip = presentDir;
        e.dstDownloadOrUnzip = futureDir;
        return e;
    }

    public static RFBHandlerEvent cdend(RFBHandler o,
                                        String presentDir,
                                        String futureDir) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.CDEND);
        e.srcDownloadOrUnzip = presentDir;
        e.dstDownloadOrUnzip = futureDir;
        return e;
    }

    public static RFBHandlerEvent ftbegin(RFBHandler o,
                                          String srcFilename,
                                          String destPath,
                                          boolean isDownload,
                                          boolean isDirectory) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.FTBEGIN);
        e.srcDownloadOrUnzip = srcFilename;
        e.dstDownloadOrUnzip = destPath;
        e.isDownload = isDownload;
        e.isDirectory = isDirectory;
        return e;
    }

    public static RFBHandlerEvent ftend(RFBHandler o,
                                        String file,
                                        String directory,
                                        boolean isDownload,
                                        boolean isDirectory,
                                        boolean isDeflating) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.FTEND);
        e.srcDownloadOrUnzip = file;
        e.dstDownloadOrUnzip = directory;
        e.isDownload = isDownload;
        e.isDirectory = isDirectory;
        e.isDeflating = isDeflating;
        return e;
    }

    public static RFBHandlerEvent ftError(RFBHandler o,
                                          String file,
                                          String directory,
                                          String errorMsg) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.FTERROR);
        e.srcDownloadOrUnzip = file;
        e.dstDownloadOrUnzip = directory;
        e.errMsg = errorMsg;
        return e;
    }

    public static RFBHandlerEvent ftprogress(RFBHandler o,
                                             String file,
                                             boolean isDownload,
                                             int progressInPercent,
                                             long bytesPerSec) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.FTPROGRESS);
        e.srcDownloadOrUnzip = file;
        e.isDownload = isDownload;
        e.progress = progressInPercent;
        e.bps = bytesPerSec;
        return e;
    }

    public static RFBHandlerEvent dircreate(RFBHandler o,
                                            String nameOfNewDirectory,
                                            String directoryCreatingIn,
                                            String errorMessage) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.DIRCREATE);
        e.srcDownloadOrUnzip = nameOfNewDirectory;
        e.dstDownloadOrUnzip = directoryCreatingIn;
        e.isDirectory = true;
        e.errMsg = errorMessage;
        return e;
    }

    public static RFBHandlerEvent filedelete(RFBHandler o,
                                             String fileDeleted,
                                             String errorMessage) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.FILEDELETE);
        e.srcDownloadOrUnzip = fileDeleted;
        e.isDirectory = false;
        e.errMsg = errorMessage;
        return e;
    }

    public static RFBHandlerEvent zipbegin(RFBHandler o,
                                           String file2Unzip,
                                           String dir2Unzip2,
                                           boolean isDirectory,
                                           boolean isDeflating) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.ZIPBEGIN);
        e.srcDownloadOrUnzip = file2Unzip;
        e.dstDownloadOrUnzip = dir2Unzip2;
        e.isDirectory = isDirectory;
        e.isDeflating = isDeflating;
        return e;
    }

    public static RFBHandlerEvent zipend(RFBHandler o,
                                         String zipFN,
                                         String zipTargetDir,
                                         String errorMessage,
                                         boolean isDownload,
                                         boolean isDirectory,
                                         boolean isDeflating) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.ZIPEND);
        e.srcDownloadOrUnzip = zipFN;
        e.dstDownloadOrUnzip = zipTargetDir;
        e.isDownload = isDownload;
        e.isDirectory = isDirectory;
        e.isDeflating = isDeflating;
        e.errMsg = errorMessage;
        return e;
    }


    public static RFBHandlerEvent zipprogress(RFBHandler o,
                                              String file2Unzip,
                                              String dir2Unzip2,
                                              boolean isDownload,
                                              boolean isDirectory,
                                              boolean isDeflating) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.ZIPPROGRESS);
        e.srcDownloadOrUnzip = file2Unzip;
        e.dstDownloadOrUnzip = dir2Unzip2;
        e.isDownload = isDownload;
        e.isDirectory = isDirectory;
        e.isDeflating = isDeflating;
        e.success = true;  // Progress messages are always successful!
        return e;
    }

    public static RFBHandlerEvent drvlistchanged(RFBHandler o) {
        return new RFBHandlerEvent(o, Category.DRVLISTCHANGED);
    }

    public static RFBHandlerEvent rfsgrew(RFBHandler o,
                                          String nameOfEntryAdded,
                                          boolean entryIsDirectory) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.RFSGREW);
        e.srcDownloadOrUnzip = nameOfEntryAdded;
        e.isDirectory = entryIsDirectory;
        return e;
    }

    public static RFBHandlerEvent pdchanged(RFBHandler o, int width, int height, boolean mouseEvent) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.PDCHANGED);
        e.isMouse = mouseEvent;
        e.cursorX = width;
        e.cursorY = height;
        return e;
    }

    public static RFBHandlerEvent cursormoved(RFBHandler o, int x, int y) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.CURSORMOVED);
        e.isMouse = true;
        e.cursorX = x;
        e.cursorY = y;
        return e;
    }

    /**
     * The key jammed event is to be sent if a key has been programmatically jammed/released.
     * At the time of writing, the only key that could be jammed, was the ALT key. As other
     * modifier keys may be made "jammable" in the future, a modifier can be passed in, which
     * will be stored in the progress variable.
     *
     * @param o        The {@link RFBHandler} object to make this Event out to.
     * @param jammed   Set true when jamming started, false when "unjammed".
     * @param modifier The keyboard modifier from the underlying {@link KeyEvent}.
     * @return The {@link RFBHandlerEvent} created, ready for shipping.
     */
    public static RFBHandlerEvent keyJamChanged(RFBHandler o, boolean jammed, int modifier) {
        RFBHandlerEvent e = new RFBHandlerEvent(o, Category.KEYJAMCHANGED);
        e.success = jammed;
        e.progress = modifier;
        return e;
    }


    private RFBHandlerEvent(RFBHandler source) {
        super(source);
    }

    private RFBHandlerEvent(RFBHandler source, Category c) {
        super(source);
        operation = c;
    }

    public void dump(PrintStream ps) {
        ps.println("-----------> VNCConnectionEvent <-----------");
        ps.println("Type        : [" + operation + "]");
        ps.println("Source      : [" + srcDownloadOrUnzip + "]");
        ps.println("Destination : [" + dstDownloadOrUnzip + "]");
        ps.println("isDownload  : [" + isDownload + "]");
        ps.println("isDirectory : [" + isDirectory + "]");
        ps.println("isDeflating : [" + isDeflating + "]");
        ps.println("isMouse     : [" + isMouse + "]");
        ps.println("Success     : [" + success + "]");
        ps.println("Progress    : [" + progress + "]");
        ps.println("BPS         : [" + bps + "]");
        ps.println("Cursor(x/y) : [(" + cursorX + "/" + cursorY + ")]");

        //if ( errMsg == null || errMsg.isEmpty()) return;
        ps.println("error Msg   : [" + errMsg + "]");
    }
}
