package com.wincornixdorf.vnc.rfb.conf;

import com.wincornixdorf.vnc.global.CharsetModel;
import com.wincornixdorf.vnc.global.VNCViewerOptions;
import com.wincornixdorf.vnc.rfb.RFB;
import com.wincornixdorf.vnc.rfb.fb.PixelFormat;
import com.wincornixdorf.vnc.rfb.fb.VNCEncoding;
import com.wincornixdorf.vnc.rfb.fb.VNCEncodingsList;

import java.util.Observable;
import java.util.Observer;

// Original Statement from an anonymous contributor to the JavaViewer:
// -----------------------Begin of quote------------------------------
// This one is not used anywhere. It serves as a reminder how to
// approach the problem of sharing the VNC options in between
// individual components.
// ------------------------End of quote-------------------------------
//
// Amen to that bro! Now it IS being used throughout the code ;)
//
// This class was long used to contain options which could be changed on the
// fly, making extending the Observable interface a pretty good idea. The
// use of this class however has changed, and by far not all members really
// have to be observable anymore.

/**
 * The SettingsCN (CN = Connection) class is used to store settings required for
 * setting up a VNCConnection. Some of these "options" will be negotiated with
 * the VNC server and may therefore change over time, which is why this class
 * extends the Observable interface. Most of these settings can be modified at
 * runtime. RFBHandler objects will observe the SettingsCN and then try to
 * negotiate the changes with the server automatically.
 */
public class SettingsCN extends Observable implements Observer {
    public static int MAXREPLAYDELAY = 20000;
    public static int MINREPLAYDELAY = 10;
    public static int DEFREPLAYDELAY = 100;

    public static int DEFCOMPRESSION = 5;
    public static int DEFJPEGQUALITY = 7;

    public static int MINSOCKETTIMEOUT = 100;
    public static int MAXSOCKETTIMEOUT = 180000;
    public static int DEFSOCKETTIMEOUT = 120000;

    private VNCViewerOptions.VNCMode mode = null;

    public CharsetModel presumedServerEncoding = null;

    private PixelFormat.PixelFormatQuality pixelFormatQuality = null;
    private boolean useCopyRect = true;

    private boolean requestCursorUpdates = false;
    private boolean ignoreCursorUpdates = false;    // used to be true

    private boolean reverseMouseButtons2And3 = false;
    private boolean shareDesktop = true;
    private boolean viewOnly = false;
    private boolean blankConsole = false;

    //private boolean         preRecordingRequestCursorUpdates    = requestCursorUpdates;
    //private boolean         preRecordingEightBitColors          = false/*eightBitColors*/;

    private int replayDelay = DEFREPLAYDELAY;

    private int socketTimeout = DEFSOCKETTIMEOUT;

    /**
     * A vector containing the encodings supported by this viewer in the order
     * the viewer prefers to use them. This list ( and some pseudo encodings)
     * will be sent to the server when setEncodings gets called.
     */
    public VNCEncodingsList preferredEncodings = new VNCEncodingsList(DEFCOMPRESSION, DEFJPEGQUALITY);

    public enum VNCProperty {
        PIXENCODING, TEXTENCODING, CANVAS, COMPRESS, JPEGQUALITY, COPYRECT, EIGHTBIT,
        UPDCURSOR, IGNORECURSOR, REVERSEMOUSE, SHAREDT, SHOWBUTTONS, VIEWONLY, BLANK,
        ISRECORDING, REPLAYDELAY, SOCKETTIMEOUT
    }

    ;

    public static Integer[] getAvailableCompQualityLevels() {
        Integer[] r = new Integer[10];
        for (int i = 0; i <= 9; i++)
            r[i] = new Integer(i);
        return (r);
    }

    public static Integer[] getAvailableJPEGQualityLevels() {
        Integer[] r = new Integer[10];
        for (int i = 0; i <= 9; i++)
            r[i] = new Integer(i - 1);
        return (r);
    }

    public SettingsCN() {
        super();
        init(null);
    }

    public SettingsCN(VNCViewerOptions o) {
        super();
        init(o);
    }

    private void init(VNCViewerOptions o) {
        presumedServerEncoding = new CharsetModel();
        presumedServerEncoding.addObserver(this);

        if (o == null) return;

        pixelFormatQuality = o.getPixelFormatQuality();
        mode = o.getAppMode();
        setBlankConsole(o.wantsConsoleBlanked());
        setPreferredEncoding(o.getDefaultEncoding());
    }

    public boolean isFBPossible() {
        return mode != VNCViewerOptions.VNCMode.FT_ONLY;
    }

    public boolean isFTPossible() {
        return mode != VNCViewerOptions.VNCMode.FB_ONLY;
    }

    public boolean isFTOnly() {
        return mode == VNCViewerOptions.VNCMode.FT_ONLY;
    }

    public boolean isFBOnly() {
        return mode == VNCViewerOptions.VNCMode.FB_ONLY;
    }

    public boolean isBothFBAndFT() {
        return (mode != VNCViewerOptions.VNCMode.FB_ONLY &&
                mode != VNCViewerOptions.VNCMode.FT_ONLY);
    }

    public VNCViewerOptions.VNCMode getVNCMode() {
        return mode;
    }

    public VNCEncoding getPreferredEncoding() {
        return (VNCEncoding) preferredEncodings.getSelectedItem();
    }

    public boolean setPreferredEncoding(VNCEncoding e) {
        preferredEncodings.setSelectedItem(e);
        setChanged();
        notifyObservers(VNCProperty.PIXENCODING);
        return true;
    }

    public int getCompressLevel() {
        return preferredEncodings.getSpecialValue(RFB.EncodingCompressLevel0);
    }

    public void setCompressLevel(int i) {
        int compressLevel = getCompressLevel();
        if (i == compressLevel || compressLevel < 0 || compressLevel > 9) return;
        preferredEncodings.setSpecialValue(RFB.EncodingCompressLevel0, i);
        setChanged();
        notifyObservers(VNCProperty.PIXENCODING);
//         notifyObservers( VNCProperty.COMPRESS);
    }

    public int getJpegQuality() {
        return preferredEncodings.getSpecialValue(RFB.EncodingQualityLevel0);
    }

    public void setJpegQuality(int i) {
        int jpegQuality = getJpegQuality();
        if (i == jpegQuality || jpegQuality < 0 || jpegQuality > 9) return;
        preferredEncodings.setSpecialValue(RFB.EncodingQualityLevel0, i);
        setChanged();
        notifyObservers(VNCProperty.PIXENCODING);
//         notifyObservers( VNCProperty.JPEGQUALITY);
    }

    public PixelFormat.PixelFormatQuality getPixelFormatQuality() {
        return pixelFormatQuality;
    }

    public void setPixFormat(PixelFormat pf) {
        if (pf == null || pf.equals(pixelFormatQuality.getFormat())) return;
        pixelFormatQuality = PixelFormat.PixelFormatQuality.getTypeByCode(pf.getBitsPerPixel());
        setChanged();
        notifyObservers(VNCProperty.EIGHTBIT);
    }

//    public int getPixFormat() { return pixFormat; }
//    public void setPixFormat( int i) {
//        if ( i == pixFormat || ( i != 1 && i != 0 && i != -1)) return;
//
//        pixFormat = i;
//        setChanged();
////         notifyObservers( VNCProperty.ENCODING);
//        notifyObservers( VNCProperty.EIGHTBIT);
//    }
//
//    public void setPixFormat( String s) {
//        if ( s == null) return;
//
//        if ( s.equalsIgnoreCase( PixelFormat.str08Bit)) setPixFormat( -1);
//        if ( s.equalsIgnoreCase( PixelFormat.str16Bit)) setPixFormat(  0);
//        if ( s.equalsIgnoreCase( PixelFormat.str32Bit)) setPixFormat(  1);
//    }
//
//    public String getPixFormatAsString() {
//        switch ( pixFormat) {
//            case -1 : return PixelFormat.str08Bit;
//            case  0 : return PixelFormat.str16Bit;
//            default : return PixelFormat.str32Bit;
//        }
//    }

    public boolean wants2UseCopyRect() {
        return useCopyRect;
    }

    public void setUseCopyRect(boolean b) {
        if (b == useCopyRect) return;
        useCopyRect = b;
        setChanged();
        notifyObservers(VNCProperty.COPYRECT);
    }

    public boolean wantsEightBitColors() {
        return pixelFormatQuality.getFormat().getBitsPerPixel() == 8;
    }

    public boolean wantsCursorUpdates() {
        return requestCursorUpdates;
    }

    public void setRequestCursorUpdates(boolean b) {
        if (b == requestCursorUpdates) return;
        requestCursorUpdates = b;
        setChanged();
        notifyObservers(VNCProperty.UPDCURSOR);
    }

    public boolean wants2IgnoreCursorUpdates() {
        return ignoreCursorUpdates;
    }

    public void setIgnoreCursorUpdates(boolean b) {
        if (b == ignoreCursorUpdates) return;
        ignoreCursorUpdates = b;
        setChanged();
        notifyObservers(VNCProperty.IGNORECURSOR);
    }

    public boolean wantsReversedMouseButtons() {
        return reverseMouseButtons2And3;
    }

    public void setReversedMouseButtons(boolean b) {
        if (b == reverseMouseButtons2And3) return;
        reverseMouseButtons2And3 = b;
        setChanged();
        notifyObservers(VNCProperty.REVERSEMOUSE);
    }

    public boolean wants2ShareDesktop() {
        return shareDesktop;
    }

    public void setShareDesktop(boolean b) {
        if (b == shareDesktop) return;
        shareDesktop = b;
        setChanged();
        notifyObservers(VNCProperty.SHAREDT);
    }

    public boolean wants2ViewOnly() {
        return viewOnly;
    }

    public void setViewOnly(boolean b) {
        if (b == viewOnly) return;
        viewOnly = b;
        setChanged();
        notifyObservers(VNCProperty.VIEWONLY);
    }

    public boolean wantsConsoleBlanked() {
        return blankConsole;
    }

    public void setBlankConsole(boolean b) {
        if (b == blankConsole) return;
        blankConsole = b;
        setChanged();
        notifyObservers(VNCProperty.BLANK);
    }

    public int getReplayDelay() {
        return replayDelay;
    }

    public void setReplayDelay(int i) {
        if (i < MINREPLAYDELAY || i > MAXREPLAYDELAY || i == replayDelay) return;
        replayDelay = i;

        setChanged();
        notifyObservers(VNCProperty.REPLAYDELAY);
    }

    public int getSocketTimeout() {
        return socketTimeout;
    }

    public void incSocketTimeout(int x) {
        if (x > 0) setSocketTimeout(socketTimeout + x);
    }

    public void setSocketTimeout(int i) {
        if (i < MINSOCKETTIMEOUT || i > MAXSOCKETTIMEOUT || i == socketTimeout) return;
        socketTimeout = i;

        setChanged();
        notifyObservers(VNCProperty.SOCKETTIMEOUT);
    }

    @Override
    public void update(Observable o, Object arg) {
        if (o != presumedServerEncoding) return;
        notifyObservers(VNCProperty.TEXTENCODING);
    }
}
