package com.wincornixdorf.vnc.rfb.decoder;

import java.io.*;
import java.awt.Color;

import com.wincornixdorf.vnc.rfb.fb.PixelFormat;
import com.wincornixdorf.vnc.rfb.fb.RemoteDesktop;
import com.wincornixdorf.vnc.rfb.fb.VNCEncoding;

/**Ideally this would be an interface rather than a class, but I wanted to have
various methods "at hand" when writing my code. For now anyone interested in
writing a Decoder of their own is kindly asked to overload the respective
methods in their subclasses without being pushed by some annoying compiler ^^*/
public class FBDecoder {

    /**Take note this is only a copy of the data. Do not share it with anyone else as it
    may become outdated at any time.*/
    protected RemoteDesktop rd;

    public FBDecoder( RemoteDesktop rd) {
        this.rd  = rd;
    }

    // TODO: make this one abstract
    /**This method has to be implemented by subclasses.
     * @return Should return the {@link VNCEncoding} the respective subclass is able to handle*/
    public VNCEncoding getEncoding() {
        return VNCEncoding.Auto;
    }

    // TODO: make this one abstract
    /**This method has to be implemented by subclasses.
     * @param isOfSession The {@link DataInputStream} to read from
     * @param x Top left corner X of the tile as stated by the Server.
     * @param y Top left corner Y of the tile as stated by the Server.
     * @param w Width of the tile as stated by the Server.
     * @param h Height of the tile as stated by the Server.
     * @throws Exception Subclasses should mainly throw {@link IOException}s where necessary.*/
    public void process( DataInputStream isOfSession, int x, int y, int w, int h) throws Exception  {
    }

    /**Reads \a nbytes from the byte array b starting from \a index, and
    attempts to interpret the bytes as an RGBA value. This method is a wee bit
    bogus. Do not use it without reflecting what it does.
    @param b The bytes to process
    @param index The position within the byte array from where to start reading
    @param nbytes The number of bytes to process.
    @return The Color encoded in the input*/
    protected Color decodeColor( byte[] b, int index, int nbytes) {
        int k = 0;
        // VNCConnection.printDbg( "decCol idx: " + index + " nbytes: " + nbytes + " size: " + b.length);
        for ( int i = index; i < index + nbytes && i < b.length; i++)
            k = k | ( b[ i] & 0xFF) << i * 8;

        // WARNING !!!!
        // Take note the original code used to 'or' (|) the bytes like this:
        // for ( int i = index; i < index + nbytes; i++)
        //     int k = k | ( b[ i]) & 0xFF) << nbytes - i * 8;
        return new Color( k, true);
    }

    protected Color decode16BitColor( byte[] buff, int index) {
        PixelFormat pf = rd.getPixelFormat();
        int b1 = ( buff[ index + 1] & 0xFF) << 8;
        int b2 = ( buff[ index] & 0xFF);

//        if ( pf.need2swap()) { int hi = b1; b1 = b2; b2 = hi; }

        int h = b1 | b2;

        int r = ( h >> pf.getRedShift())   & pf.getRedMax();
        int g = ( h >> pf.getGreenShift()) & pf.getGreenMax();
        int b = ( h >> pf.getBlueShift())  & pf.getBlueMax();

        double fr = pf.getRedMax()   != 0 ? ( r * 1.0) / ( pf.getRedMax()   * 1.0) : 0.0;
        double fg = pf.getGreenMax() != 0 ? ( g * 1.0) / ( pf.getGreenMax() * 1.0) : 0.0;
        double fb = pf.getBlueMax()  != 0 ? ( b * 1.0) / ( pf.getBlueMax()  * 1.0) : 0.0;

        return new Color( (float)fr, (float)fg, (float)fb);
    }

    protected static Color decodeColor2( byte[] b, int index, int nbytes) {
        int k = 0; int l = 2;
        for ( int i = index; i < index + nbytes  && i < b.length; i++)
            k = k | ( b[ i] & 0xFF) << ( l--) * 8;
        return new Color( k, true);
    }
    
    protected int crunch( byte[] b) {
        int k = 0;
        for ( int i = 0; i < b.length; i++) k = k | ( b[ i] & 0xFF) << i * 8;
        return k;
    }

//    protected int crunch( byte[] b, int index, int nbytes) {
//        int k = 0, j = 0;
//        for ( int i = index; i < index + nbytes && i < b.length; i++)
//            k = k | ( b[ i] & 0xFF) << j * 8;
//
//        return k;
//    }

    protected Color decode16BitColor5( byte[] buf) {
        return new Color(0xFF000000 | (buf[0] & 0xFF) << 16 |
                 (buf[1] & 0xFF) << 8 | (buf[2] & 0xFF));
    }

    protected String dbgTile( int x, int y, int w, int h, int bpp) {
        return "Tile (x/y/w/h/bpp) = (" + x + "/" + y + "/" + w + "/" + h + "/" + bpp + ")";
    }

    /**The base class is able to handle RAW encoded frames as some of the subclasses require
     * this functionality they themselves.
     * @param is The {@link DataInputStream} to read from
     * @param x Top left corner X of the tile as stated by the Server.
     * @param y Top left corner Y of the tile as stated by the Server.
     * @param w Width of the tile as stated by the Server.
     * @param h Height of the tile as stated by the Server.
     * @param bpp Color depth as stated by the Server.
     * @throws IOException when failing to read from the {@link DataInputStream}.*/
    protected void processRAW( DataInputStream is, int x, int y, int w, int h, int bpp) throws IOException {
        //System.err.println( "RR x[" + x + "] y[" + y + "] w[" + w + "] h[" + h + "] @ [" + rd.getBytesPerPixel() + "] bytes pp deph [ " + rd.getPixelFormat().getDepth() + "]");

        byte buf[] = new byte[ w * bpp];
    	
//         int bpp = rd.getBytesPerPixel();

        switch ( bpp) {
        	case 1 :
                for ( int dy = y; dy < y + h; dy++) {
                    //VncViewer.pen.log( Level.INFO, "dy " + dy + " fbw " + rd.getFBW() + " x " + x + " w " + w);
//                    is.readFully( rd.pixels08, dy * rd.getFBW() + x, w);
                	int offset = dy * rd.getFBW() + x;
                    is.readFully( buf);
                	for ( int i = 0; i < buf.length; i++) {
                		int b = buf[ i] & 0xFF;
                		Color c = new Color( rd.cm08.getRed( b) , rd.cm08.getGreen( b), rd.cm08.getBlue( b));
                		
                        rd.pixels24[ i + offset] = c.getRGB();
                	}
                    // if (rfb.rec != null)
                    //	rfb.rec.write(pixels08, dy * rfb.framebufferWidth + x, w);
                }        		
        	break;
        	case 2:
                for ( int dy = y; dy < y + h; dy++) {
                    int offset = dy * rd.getFBW() + x;
                    is.readFully( buf);
                    for ( int i = 0; i < w; i++)
                        rd.pixels24[ i + offset] = decode16BitColor( buf, i * bpp).getRGB();
                }
        	break;
        	default:
                for ( int dy = y; dy < y + h; dy++) {
                    int offset = dy * rd.getFBW() + x;
                    is.readFully( buf);
                    for ( int i = 0; i < w; i++)
                        rd.pixels24[ i + offset] = decodeColor( buf, i * bpp, bpp).getRGB();
                }
            break;
        }

        rd.srcRDesktop.newPixels( x, y, w, h);
    }
}
