package com.wincornixdorf.vnc.rfb.decoder;

import java.awt.Color;
import java.io.DataInputStream;

import com.wincornixdorf.vnc.rfb.fb.RemoteDesktop;
import com.wincornixdorf.vnc.rfb.fb.VNCEncoding;
import com.wincornixdorf.vnc.rfb.stream.InStream;
import com.wincornixdorf.vnc.rfb.stream.MemInStream;
import com.wincornixdorf.vnc.rfb.stream.ZlibInStream;

public class FBDecoderZRLE extends FBDecoder {
    private ZlibInStream zrleInStream	= new ZlibInStream();

    private int zrleBufLen              = 0;

    private byte[] zrleBuf              = null;

    private int[] zrleTilePixels24      = new int[ 64 * 64];

    public FBDecoderZRLE( RemoteDesktop rd) {
        super( rd);
    }

    public VNCEncoding getEncoding() {
        return VNCEncoding.ZRLE;
    }

    public void process( DataInputStream is, int x, int y, int w, int h) throws Exception {
        int nBytes = is.readInt();
        if ( nBytes > 64 * 1024 * 1024)
            throw new Exception( "ZRLE decoder: illegal compressed data size");

        if ( zrleBuf == null || zrleBufLen < nBytes) {
            zrleBufLen = nBytes + 4096;
            zrleBuf = new byte[ zrleBufLen];
        }

        // FIXME: Do not wait for all the data before decompression.
        is.readFully( zrleBuf, 0, nBytes);

        //if (rfb.rec != null) {
        //    if (rfb.recordFromBeginning) {
        //        rfb.rec.writeIntBE(nBytes);
        //        rfb.rec.write(zrleBuf, 0, nBytes);
        //    } else if ( !zrleRecWarningShown) {
        //        System.out.println("Warning: ZRLE session can be recorded" +
        //                        " only from the beginning");
        //        System.out.println("Warning: Recorded file may be corrupted");
        //        zrleRecWarningShown = true;
        //    }
        //}

        zrleInStream.setUnderlying( new MemInStream( zrleBuf, 0, nBytes), nBytes);

        for ( int ty = y; ty < y + h; ty += 64) {
            int th = Math.min( y + h - ty, 64);

            for ( int tx = x; tx < x + w; tx += 64) {
                int tw = Math.min( x + w - tx, 64);

                int mode = zrleInStream.readU8();
                boolean rle = ( mode & 128) != 0;
                int palSize = mode & 127;
                int[] palette = new int[ 128];

                readZrlePalette( palette, palSize);

                //System.out.println( "PalSize: " + palSize + " RLE: " + rle + " th: " + th + " tw: " + tw);

                if ( palSize == 1) {
                    int pix = palette[ 0];
                    Color c = new Color( rd.getBytesPerPixel() == 1 ? rd.cm08.getRGB( pix) : 0xFF000000 | pix);
                    rd.fillRect( tx, ty, tw, th, c);
                    continue;
                }

                if ( !rle) {
                    if ( palSize == 0) readZrleRawPixels( tw, th);
                    else readZrlePackedPixels( tw, th, palette, palSize);
                } else {
                    if ( palSize == 0) readZrlePlainRLEPixels( tw, th);
                    else readZrlePackedRLEPixels( tw, th, palette);
                }

                handleUpdatedZrleTile( tx, ty, tw, th);
            }
        }

        zrleInStream.reset();
        rd.srcRDesktop.newPixels( x, y, w, h);
    }

    private int readPixel( InStream is) throws Exception {
        int bpp = rd.getBytesPerPixel();
        int cs  = rd.getCrunchSize();

        byte[] buf = new byte[ cs];
        is.readBytes( buf, 0, buf.length);

        switch ( bpp) {
            case 1  : return buf[ 0];
            case 2  : return decode16BitColor( buf, 0).getRGB();
            default : return crunch( buf);
        }
    }

    private void readPixels( InStream is, int[] dst, int count) throws Exception {
        int bpp = rd.getBytesPerPixel();
        int cs  = rd.getCrunchSize();

        byte[] buf = new byte[ cs];

        switch ( bpp) {
            case 1  : for ( int i = 0; i < count; i++) dst[ i] = is.readU8(); break;
            case 2  :
                for ( int i = 0; i < count; i++) {
                    is.readBytes( buf, 0, buf.length);
                    dst[ i] = decode16BitColor( buf, 0).getRGB();
                }
            break;
            default :
                for ( int i = 0; i < count; i++) {
                    is.readBytes( buf, 0, buf.length);
                    dst[ i] = crunch( buf);
                }
            break;
        }
    }

    private void readZrlePalette( int[] palette, int palSize) throws Exception {
        readPixels( zrleInStream, palette, palSize);
    }

    private void readZrleRawPixels( int tw, int th) throws Exception {
        readPixels( zrleInStream, zrleTilePixels24, tw * th);
    }

    private void readZrlePackedPixels( int tw, int th, int[] palette, int palSize) throws Exception {
        int bppp = (( palSize > 16) ? 8 : (( palSize > 4) ? 4 : (( palSize > 2) ? 2 : 1)));
        int ptr = 0;

        for ( int i = 0; i < th; i++) {
            int eol = ptr + tw;
            int b = 0;
            int nbits = 0;

            while ( ptr < eol) {
                if ( nbits == 0) {
                    b = zrleInStream.readU8();
                    nbits = 8;
                }

                nbits -= bppp;
                int index = ( b >> nbits) & (( 1 << bppp) - 1) & 127;

                if ( rd.getBytesPerPixel() == 1)
                	zrleTilePixels24[ ptr++] = rd.cm08.getRGB( palette[ index]);
                else
                    zrleTilePixels24[ ptr++] = palette[ index];
            }
        }
    }

    private void readZrlePlainRLEPixels( int tw, int th) throws Exception {
        int ptr = 0;
        int end = ptr + tw * th;

        while ( ptr < end) {
            int pix = readPixel( zrleInStream);
            int len = 1;
            int b;

            do {
                b = zrleInStream.readU8();
                len += b;
            } while ( b == 255);

            if ( !( len <= end - ptr))
                throw new Exception( "ZRLE decoder: assertion failed (len <= end-ptr)");

            if ( rd.getBytesPerPixel() == 1) {
            	while ( len-- > 0) zrleTilePixels24[ ptr++] = rd.cm08.getRGB( pix);
            } else {
                while ( len-- > 0) zrleTilePixels24[ ptr++] = pix;
            }
        }
    }

    private void readZrlePackedRLEPixels( int tw, int th, int[] palette) throws Exception {
        int ptr = 0;
        int end = ptr + tw * th;

        while ( ptr < end) {
            int index = zrleInStream.readU8();
            int len = 1;

            //System.out.println( "idx : " + index + " len: " + len + " ptr: " + ptr + " end: " + end);

            if (( index & 128) != 0) {
                int b;
                do {
                    b = zrleInStream.readU8();
                    len += b;
                } while ( b == 255);

                if ( !( len <= end - ptr))
                    throw new Exception( "ZRLE decoder: assertion failed (len <= end - ptr)");
            }

            index &= 127;
            int pix = palette[ index];

            if ( rd.getBytesPerPixel() == 1) {
            	while ( len-- > 0) zrleTilePixels24[ ptr++] = rd.cm08.getRGB( pix);
            } else {
                while ( len-- > 0) zrleTilePixels24[ ptr++] = pix;
            }
        }
    }

    // Copy pixels from zrleTilePixels8 or zrleTilePixels24, then update.
    private void handleUpdatedZrleTile( int x, int y, int w, int h) {
        Object src = zrleTilePixels24, dst = rd.pixels24;
        
        int offsetSrc = 0;
        int offsetDst = ( y * rd.getFBW() + x);

        for ( int j = 0; j < h; j++) {
            System.arraycopy( src, offsetSrc, dst, offsetDst, w);
            offsetSrc += w;
            offsetDst += rd.getFBW();
        }
    }
}
