package com.wincornixdorf.vnc.rfb.fb;

import java.util.Observable;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.ColorModel;
import java.awt.image.MemoryImageSource;
import java.io.DataInputStream;
import java.io.IOException;

public class RemoteCursor extends Observable
{
	private static Dimension DEFAULT_SIZE = new Dimension( 24, 24);

    /**Cursors position using the remote desktop's coordinate system.*/
    private Point pRemote           = new Point();

    /**This rectangles size reflects the the cursor size and hot spot
    in its 'location' and its 'size'*/
    private Rectangle r             = new Rectangle( 1, 1, DEFAULT_SIZE.width, DEFAULT_SIZE.height);

    private MemoryImageSource src   = null;
    private int[] pixels            = null;

    /**A private color Table only used internally.*/
    private byte[] rgb              = new byte[ 6];

    public RemoteCursor() {
    	allocateMemory( r.width, r.height);
    }

    public Point getRemotePos() {
        return pRemote;
    }

    public void setRemotePos( Point p) {
        pRemote = p;
    }

    public Point getHotSpot() {
        return r.getLocation();
    }

    public Dimension getSize() {
        return r.getSize();
    }

    public Rectangle getRect() {
        return r;
    }

    public MemoryImageSource getImageSource() {
        return src;
    }

    /**Reads the shape of the new Cursor from the {@link DataInputStream}, using the provided
     * {@link ColorModel} when decoding colours. This functionality however is known to have
     * a bug causing off-colours.
     * @param is The {@link DataInputStream} to read from.
     * @param cm The {@link ColorModel} to use as reference when decoding colours.
     * @param encodingType The {@link VNCEncoding} to assume when decoding the cursor.
     * @param xhot Cursor hot-spot X coordinate as posted by the server.
     * @param yhot Cursor hot-spot Y coordinate as posted by the server.
     * @param width Cursor width as posted by the server.
     * @param height Cursor height as posted by the server.
     * @param bytesPerPixel The colour depth to presume.
     * @throws IOException if the {@link DataInputStream} can't be read from.*/
    public void readNewCursorShape(
            DataInputStream is,
            ColorModel cm,
            VNCEncoding encodingType,
            int xhot,
            int yhot,
            int width,
            int height,
            int bytesPerPixel) throws IOException
    {
 		//VNCViewer.pen.finest( "et:" + encodingType.toString() + " xh:" + xhot + " yh:" + yhot + " w:" + width + " h:" + height + " bpp:" + bytesPerPixel);

        if ( width * height == 0) return;

        r.setLocation( xhot, yhot);
        allocateMemory( width, height);

        // Decode cursor pixel data.
        decodeCursorShape( is, cm, encodingType, width, height, bytesPerPixel);
    }

    private void decodeCursorShape(
                DataInputStream is,
                ColorModel cm,
                VNCEncoding encodingType,
                int width,
                int height,
                int bytesPerPixel) throws IOException
    {
        int bytesPerRow   = ( width + 7) / 8;
        int bytesMaskData = bytesPerRow * height;

 		//VNCViewer.pen.finest( "et:" + encodingType.toString() + " w:" + width + " h:" + height + " bpp:" + bytesPerPixel + " cm:" + cm.toString());

 		
        if ( encodingType == VNCEncoding.XCursor) {
            // Read foreground and background colors of the cursor.
            is.readFully( rgb);
            int[] colors = { (0xFF000000 | ( rgb[ 3] & 0xFF) << 16 |
                                           ( rgb[ 4] & 0xFF) <<  8 |
                                           ( rgb[ 5] & 0xFF)),
                             (0xFF000000 | ( rgb[ 0] & 0xFF) << 16 |
                                           ( rgb[ 1] & 0xFF) <<  8 |
                                           ( rgb[ 2] & 0xFF)) };


            // Read pixel and mask data.
            byte[] pixBuf = new byte[ bytesMaskData];
            is.readFully( pixBuf);
            byte[] maskBuf = new byte[ bytesMaskData];
            is.readFully( maskBuf);
            
            // Decode pixel data into pixels[].
            byte pixByte, maskByte;
            int x, y, n, result;
            int i = 0;
            for ( y = 0; y < height; y++) {
                for ( x = 0; x < width / 8; x++) {
                    pixByte = pixBuf[ y * bytesPerRow + x];
                    maskByte = maskBuf[ y * bytesPerRow + x];
                    for ( n = 7; n >= 0; n--) {
                        // result of 0 means transparent pixel
                        pixels[ i++] = ( maskByte >> n & 1) != 0 ? colors[ pixByte >> n & 1] : 0;
                    }
                }
                for (n = 7; n >= 8 - width % 8; n--) {
                    if ( (maskBuf[ y * bytesPerRow + x] >> n & 1) != 0)
                        result = colors[ pixBuf[ y * bytesPerRow + x] >> n & 1];
                    else
                        result = 0;		// Transparent pixel
                    pixels[ i++] = result;
                }
            }
        } else {    		
            // Read pixel and mask data.
            byte[] pixBuf = new byte[ width * height * bytesPerPixel];
            is.readFully( pixBuf);
            byte[] maskBuf = new byte[ bytesMaskData];
            is.readFully( maskBuf);

            // Decode pixel data into pixels[].
            byte maskByte;
            int x, y, n, result;
            int i = 0;

            for ( y = 0; y < height; y++) {
                for ( x = 0; x < width / 8; x++) {
                    maskByte = maskBuf[ y * bytesPerRow + x];
                    for ( n = 7; n >= 0; n--) {
                        if (( maskByte >> n & 1) != 0) {
//                            if ( bytesPerPixel == 1)
//                                result = cm.getRGB( pixBuf[ i]);
//                            else
////                                result = 0xFF000000 | ( pixBuf[ i * 4 + 2] & 0xFF) << 16 |
////                                                      ( pixBuf[ i * 4 + 1] & 0xFF) <<  8 |
////                                                      ( pixBuf[ i * 4    ] & 0xFF);
//							result =
//									0xFF000000 | (pixBuf[i * 4 + 1] & 0xFF)
//                                         << 16 | (pixBuf[i * 4 + 2] & 0xFF)
//                                         << 8  | (pixBuf[i * 4 + 3] & 0xFF);

                        	switch ( bytesPerPixel) {
                        		case 1:  result = cm.getRGB( pixBuf[ i] & 0xFF); break;
                        		case 2:  result = 0xFF000000 | ( pixBuf[ i * 2 + 1] & 0xFF) <<  8 |
                                                               ( pixBuf[ i * 2    ] & 0xFF);
                        		break;
                        		default: result = 0xFF000000 | ( pixBuf[ i * 4 + 2] & 0xFF) << 16 |
                                                               ( pixBuf[ i * 4 + 1] & 0xFF) <<  8 |
                                                               ( pixBuf[ i * 4    ] & 0xFF);
                        		break;
                        		
                        	}
                        } else result = 0;	// Transparent pixel
                        pixels[ i++] = result;
                    }
                }
                for ( n = 7; n >= 8 - width % 8; n--) {
                    if (( maskBuf[ y * bytesPerRow + x] >> n & 1) != 0) {
//                        if ( bytesPerPixel == 1)
//                            result = cm.getRGB( pixBuf[ i]);
//                        else
//							result =
//							0xFF000000 | (pixBuf[i * 4 + 1] & 0xFF)
//								<< 16 | (pixBuf[i * 4 + 2] & 0xFF)
//								<< 8 | (pixBuf[i * 4 + 3] & 0xFF);
//                            result = 0xFF000000 | ( pixBuf[ i * 4 + 2] & 0xFF) << 16 |
//                                                  ( pixBuf[ i * 4 + 1] & 0xFF) <<  8 |
//                                                  ( pixBuf[ i * 4    ] & 0xFF);
                    	switch ( bytesPerPixel) {
	                		case 1:  result = cm.getRGB( pixBuf[ i]); break;
	                		case 2:  result = 0xFF000000 | ( pixBuf[ i * 2 + 1] & 0xFF) <<  8 |
	                                                       ( pixBuf[ i * 2    ] & 0xFF);
	                		break;
	                		default: result = 0xFF000000 | ( pixBuf[ i * 4 + 2] & 0xFF) << 16 |
                                                           ( pixBuf[ i * 4 + 1] & 0xFF) <<  8 |
                                                           ( pixBuf[ i * 4    ] & 0xFF);
	                		break;                		
                    	}                   	
                    } else result = 0;	// Transparent pixel
                    pixels[ i++] = result;
                }
            }
        }
    }

    /**Sets the size of the remote cursor and ensures the pixel buffer backing the
     * image the cursor is created from is the right size. If either width or height
     * are less or equal to zero, a default 24*24 lumb of memory is allocated.
     * @param width The width the new cursor is to have.
     * @param height The height the new cursor is to have.*/
    private void allocateMemory( int width, int height) { 	
    	if ( width <= 0 || height <= 0) {
    		width  = DEFAULT_SIZE.width;
    		height = DEFAULT_SIZE.height;
    	}

    	r.setSize( width, height);
    	int sizeNewBuffer = width * height;

        if ( pixels != null && pixels.length == sizeNewBuffer) return;

        pixels = new int[ sizeNewBuffer];

        int mw = width  / 2;
        int mh = height / 2;
        int mm = Math.min( mw, mh) - 2; 
        int m = (mh * width + mw);
        int d = Color.white.getRGB();
        
        for ( int ticks = mm; ticks > 0; ticks --) {
        	// horizontal
        	pixels[ m - ticks - 1] = Color.gray.getRGB();
        	pixels[ m + ticks + 1] = Color.cyan.getRGB();

        	// vertical
        	pixels[ m - ticks * width] = d;
        	pixels[ m + ticks * width] = d;
        }

        pixels[ m] = Color.red.getRGB();
//        r.setLocation( mw, mh);

//        pixels[ mh * width + mw] = Color.red.getRGB();

    	// initialise a minimalistic single point Pointer
//    	pixels[ (r.height * r.width) / 2] = d;

        src = new MemoryImageSource( r.width, r.height, pixels, 0, r.width);

        //src.setAnimated( true);
        src.setFullBufferUpdates( false);
    }
}

