package com.wincornixdorf.vnc.rfb.fb;

import java.io.ByteArrayInputStream;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.image.ColorModel;
import java.awt.image.DirectColorModel;
import java.awt.image.MemoryImageSource;
import java.awt.image.BufferedImage;
import java.util.Arrays;
import java.util.Observable;

import com.wincornixdorf.vnc.global.VNCViewer;


/**The RemoteDesktop serves as a container for the low level pixel data
of the remote desktop and mouse cursor. It is mainly used to keep the
raw pixel data for reference in between different package private classes.
This class does not alter the raw data in any way. If anyone wants to e.g.
zoom the remote framebuffer, they have to do so they themselves. Whatever
it is they want to do, they are NOT to modify the raw data stored here.*/
public class RemoteDesktop extends Observable {

    /**The mouse cursor on the remote desktop. This object holds both the
    coordinates/size and the pixels of the cursor.*/
    public RemoteCursor         remoteCursor        = new RemoteCursor();

    /**The pixel format as negotiated with the server.*/
    private PixelFormat         pixFormat           = new PixelFormat();

    /**Low level pixel data for 24Bit per pixel encodings.*/
    public int[]                pixels24            = null;

    public MemoryImageSource    srcRDesktop         = null;

    /**The ORIGINAL size of the remote framebuffer. */
    private Dimension           dimFB               = new Dimension( 800, 600);

    /**The name of the desktop as sent by the server during handshake.*/
    private String              desktopName         = "unknown";

    public ColorModel           cm08                = new DirectColorModel(  8, 7, (7 << 3), (3 << 6));
    public ColorModel           cm24                = new DirectColorModel( 24, 0xFF0000, 0x00FF00, 0x0000FF);

    public ColorModel           cm8_64c 			= new DirectColorModel( 8, (3 << 4), (3 << 2), (3 << 0));
    //    public ColorModel           cm8_8c				= new DirectColorModel( 8, (1 << 2), (1 << 1), (1 << 0));
    
    public Color[]              fixedPalette        = new Color[ 256];

    public enum RDProperty { DIMENSION, DSKTNAME, DSKTIMAGESOURCE, PIXFORMAT};

    public RemoteDesktop() {
        for ( int i = 0; i < fixedPalette.length; i++)
            fixedPalette[ i] = new Color( cm24.getRGB( i));
        resizeOffscreenFramebuffer();
    }

    public void fillRect24( int x, int y, int w, int h, Color c) {
        int offset   = y * dimFB.width + x;

        int[] srcArray = new int[ w];
        int   cc       = c.getRGB();

        Arrays.fill( srcArray, cc);

        for ( int p = offset; p < offset + h * dimFB.width; p += dimFB.width)
            System.arraycopy( srcArray, 0, pixels24, p, w);
    }

    public void fillRect( int x, int y, int w, int h, Color c) {
        fillRect24( x, y, w, h, c);
    }


    public static String topUp( String s, int max) {
        StringBuilder sb = new StringBuilder();
        int sLen = s.length();

        while ( sLen < max) {
            sb.append( "0");
            sLen++;
        }

        sb.append( s);
        return sb.toString();
    }

    public static void show( int i) {
        System.out.println( "Value      : [" + i                                      + "]");
        System.out.println( "Int binary : [" + topUp( Integer.toBinaryString( i), 32) + "]");
        System.out.println( "");
    }

    public void writeJPEGData( byte[] b, int x, int y, int w, int h) {
        try {
            ByteArrayInputStream bais = new ByteArrayInputStream( b);
            BufferedImage image = javax.imageio.ImageIO.read( bais);
            int fbw = (int)dimFB.getWidth();
            int offset = y * fbw + x;

            if ( pixels24 == null || pixels24.length < ( offset + (( h + 1) * dimFB.width)))
            	return;

            int[] aLine = new int[ image.getWidth()];
            
            for ( int yy = 0; yy < image.getHeight(); yy++) {
            	image.getRGB( 0, yy, image.getWidth(), 1, aLine, 0, fbw);
            	System.arraycopy( aLine, 0, pixels24, offset + yy * fbw, image.getWidth());
            }

            // AND THAT'S WHY YOU USE arraycopy !
            // for ( int yy = 0; yy < image.getHeight(); yy++) {
            //      for ( int xx = 0; xx < image.getWidth(); xx++) {
            //      	pixels24[ offset + xx + yy * fbw] = image.getRGB( xx, yy);
            //      }
            // }

        } catch ( java.io.IOException ioe) {
            System.err.println( "No can do sir");
        }
    }

    public int getBitsPerPixel() {
        return pixFormat.getBitsPerPixel();
    }

    public int getBytesPerPixel() {
        return pixFormat.getBytesPerPixel();
    }

    public int getCrunchSize() {
        int h = pixFormat.getBytesPerPixel();
        return h > 2 ? 3 : h;
    }

    /**This one serves internal testing purposes ONLY. Do not use it!
     * @param newSetting Use current pixFormat value or previous value.
     * @return The Number of Bytes per Pixel supposedly used by the server.*/
    public int getBytesPerPixel( boolean newSetting) {
        int old = pixFormat.getBytesPerPixel() == 4 ? 1 : 4;
        return newSetting ? pixFormat.getBytesPerPixel() : old;
    }

    /**Set the remote desktop's original size. This method is also responsible for allocating
     * the memory required to house the pixel data.
     * @param d The new {@link Dimension}s to use. 
     * @param inhibitOffscreenResize If set true, the internal memory allocated to keep the
     * remote desktop's pixel data is NOT adjusted accordingly.*/
    public void setFBDim( Dimension d, boolean inhibitOffscreenResize) {
        if ( d == null || d.equals( dimFB)) return;
        dimFB = d;
        if ( !inhibitOffscreenResize) resizeOffscreenFramebuffer();

        setChanged();
        notifyObservers( RDProperty.DIMENSION);
    }

    public Dimension getFBDim() {
        return dimFB;
    }

    public int getFBW() {
        return (int)dimFB.getWidth();
    }

    public int getFBH() {
        return (int)dimFB.getHeight();
    }

    public void setDesktopName( String n) {
        if ( n == null || desktopName == n) return;
        desktopName = n;
        setChanged();
        notifyObservers( RDProperty.DSKTNAME);
    }

    public String getDesktopName() {
        return desktopName;
    }

    public boolean isLoColor() {
        return pixFormat.getBitsPerPixel() == 8;
    }

    public void setPixelFormat( PixelFormat pf) {
        if ( pf == null || pixFormat.equals( pf)) return;
        pixFormat = pf;

        dump();

        setChanged();
        notifyObservers( RDProperty.PIXFORMAT);
    }

    public PixelFormat getPixelFormat() {
    	return pixFormat;
    }

    public void dump() {
        System.out.println( "Remote Desktop Properties");
        System.out.println( "-------------------------");
        System.out.println( "RD Name                  : [" + desktopName + "]");
        System.out.println( "Resolution               : [" + dimFB.width + " x " + dimFB.height + "] @ [" + pixFormat.getBitsPerPixel() + "] Bits per Pixel");
        pixFormat.dump();
        Rectangle rc = remoteCursor.getRect();
        System.out.println( "mouse(X/Y/W/H)           : (" + rc.x + "/" + rc.y + "/" + rc.width + "/" + rc.height + ")");
    }

    private void resizeOffscreenFramebuffer() {
        int fbw = (int)dimFB.getWidth();
        int fbh = (int)dimFB.getHeight();

        VNCViewer.pen.finest( "RemoteDesktop is resizing its offscreen buffer!");
        
        pixels24 = new int[ fbw * fbh];
        srcRDesktop = new MemoryImageSource( fbw, fbh, cm24, pixels24, 0, fbw);

        srcRDesktop.setAnimated( true);
        srcRDesktop.setFullBufferUpdates( false);

        setChanged();
        notifyObservers( RDProperty.DSKTIMAGESOURCE);
    }
}
