package com.wincornixdorf.vnc.rfb.fb;

import java.util.Vector;
import java.util.Iterator;
import javax.swing.ComboBoxModel;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;


public class VNCEncodingsList implements ComboBoxModel<VNCEncoding> {
    /**A vector containing the encodings supported by this viewer in the order
    the viewer prefers to use them. This list ( and some pseudo encodings)
    will be sent to the server when setEncodings gets called.*/
    private Vector<VNCEncoding>         encodings           = null;

    private Vector<ListDataListener>    dataListeners       = null;

    private int                         encodingsOffered    = 0;
    //private int                         encodingsHidden     = 0;

    public VNCEncodingsList( int compressLevel, int jpegQuality) {
        encodings     = new Vector<VNCEncoding>();
        dataListeners = new Vector<ListDataListener>();

        /*WARNING! If you want to make sure a particular encoding gets used per
        default, set it in the VNCViewerOptions. Changing the order of these
        entries will accomplish NOTHING!*/
        encodings.add( VNCEncoding.Auto);       // Should select the best encoding
        encodings.add( VNCEncoding.Tight);      // Fast but not always supported
        encodings.add( VNCEncoding.RRE);        // Decent speed. Nothing fancy
        encodings.add( VNCEncoding.Hextile);    // Good tradeoff speed/CPU time client
        encodings.add( VNCEncoding.Zlib);       // Good compression heavy on CPU usage
        encodings.add( VNCEncoding.CoRRE);      // Fast but it doesn't always work
        encodings.add( VNCEncoding.ZRLE);       // Never seen it working ;/
        encodings.add( VNCEncoding.CopyRect);   // Seems to cause errors. Was marked !supported
        encodings.add( VNCEncoding.Raw);        // Going toe to toe with bits :(

        // May seem silly, but I would like to be able to flip encodings on
        // and off where they are being defined: In the VNCEncoding class.
        Iterator<VNCEncoding> i = encodings.iterator();
        while ( i.hasNext()) {
            VNCEncoding e = i.next();
            if ( e.isPseudo() || !e.isSupported()) i.remove();
        }

        encodingsOffered = encodings.size();

        // These two signal the quality of the compression we expect in
        // the event we use an encoding that sends JPG encoded RFB packets.
        encodings.add( VNCEncoding.craftCompressionLvl( compressLevel));
        encodings.add( VNCEncoding.craftQualityLvl( jpegQuality));
//         encodings.add( VNCEncoding.craftCompressionLvl( 6));
//         encodings.add( VNCEncoding.craftQualityLvl( 6));

        // Tell the server we would like to be sent a bitmask with the
        // remote desktops mouse cursor in it, so we can draw that on our
        // desktop to boost the perceived speed
//         if ( wantsCursorUpdates()) {
            encodings.add( VNCEncoding.XCursor);
            encodings.add( VNCEncoding.RichCursor);
//         }

        // Let the server know we would like to be informed if the remote
        // cursors position changes.
//         if ( !wants2IgnoreCursorUpdates())
            encodings.add( VNCEncoding.PointerPos);

        // Signals the server we are able to handle the LastRect signal.
        // On reading this, the server will stop sending empty FB frames
        // to pad transmissions to fit the standard FB size. Instead the
        // server will signal the client when to stop reading the stream.
        encodings.add( VNCEncoding.LastRect);

        // Signals the server we are able to handle resize events in the
        // servers FB. E.g. someone changes the resolution of the remote
        // desktop we follow suit and adjust our display accordingly.
        encodings.add( VNCEncoding.NewFBSize);

        //encodingsHidden = encodings.size() - encodingsOffered;
    }

    /**Queries the list of known {@link VNCEncoding}s for encodings of a particular type. 
     * @param includingPseudoEncodings  If set false, encodings which have been marked as
     * pseudo encodings will be omitted.  
     * @param includingPointerPos  If set false, the PointerPos encoding will be omitted.
     * @return A copy of the Encodings presently set in this list.
    */
    @SuppressWarnings("unchecked")
    public Vector<VNCEncoding> getEncodings( boolean includingPseudoEncodings,
                                             boolean includingPointerPos) {
        Vector<VNCEncoding> v = (Vector<VNCEncoding>)encodings.clone();

        if ( !includingPseudoEncodings) {
            Iterator<VNCEncoding> i = v.iterator();
            while ( i.hasNext()) {
                VNCEncoding e = i.next();
                if ( e.isPseudo()) i.remove();
            }
        }

        if ( !includingPointerPos)
            v.remove( VNCEncoding.PointerPos);

        return v;
    }

    public Object getSelectedItem() {
        return encodings.firstElement();
    }

    public void setSelectedItem( Object anItem) {
        // new item is not a VNCEncoding? Bail out!
        if ( !(anItem instanceof VNCEncoding)) return;

        // Pseudo VNCEncodings don't qualify either
        VNCEncoding e = (VNCEncoding)anItem;
        if ( e.isPseudo() || e == getSelectedItem()) return;

        // the first VNCEncoding in the list is always the selected one
        boolean noSuchItem = !encodings.remove( anItem);
        if ( noSuchItem) return;
        encodings.insertElementAt( (VNCEncoding)anItem, 0);
        /**Todo: check if this one was really a bug or not.*/
        //notifyListeners( ListDataEvent.CONTENTS_CHANGED, 0, encodingsOffered);
    }

    public boolean setSpecialValue( int encodingType, int value) {
        VNCEncoding e = getEncoding( encodingType);
        return e == null ? false : e.setSpecialValue( value);
    }

    public int getSpecialValue( int encodingType) {
        VNCEncoding e = getEncoding( encodingType);
        return e == null ? -1 : e.getSpecialValue();
    }

    public VNCEncoding getElementAt( int index) {
        return encodings.get( index);
    }

    public int getSize() {
        return encodingsOffered;
    }

    protected VNCEncoding getEncoding( int type) {
        VNCEncoding rval = null;
        Iterator<VNCEncoding> i = encodings.iterator();
        while ( rval == null && i.hasNext()) {
            VNCEncoding e = i.next();
            rval = e.getType() == type ? e : null;
        }
        return rval;
    }

    public void addListDataListener( ListDataListener l) { dataListeners.add( l); }
    public void removeListDataListener( ListDataListener l) { dataListeners.remove( l); }

    protected void notifyListeners( int evtType, int idx0, int idx1) {
        Iterator<ListDataListener> i = dataListeners.iterator();
        while ( i.hasNext()) {
            ListDataListener l = i.next();
            l.contentsChanged( new ListDataEvent( this, evtType, idx0, idx1));
        }
    }
}
